@@ -1,20 +1,21 @@
+#![allow(unused, non_upper_case_globals)]
 //! Diffing Tests
 //!
 //! These tests only verify that the diffing algorithm works properly for single components.
 //!
 //! It does not validated that component lifecycles work properly. This is done in another test file.
 
-use dioxus::{nodes::VSuspended, prelude::*, DomEdit, TestDom};
+use dioxus::{prelude::*, DomEdit};
 use dioxus_core as dioxus;
 use dioxus_core_macro::*;
 use dioxus_html as dioxus_elements;
 
 mod test_logging;
 
-fn new_dom() -> TestDom {
+fn new_dom() -> VirtualDom {
     const IS_LOGGING_ENABLED: bool = false;
     test_logging::set_up_logging(IS_LOGGING_ENABLED);
-    TestDom::new()
+    VirtualDom::new(|cx| rsx!(cx, "hi"))
 }
 
 use DomEdit::*;
@@ -23,16 +24,19 @@ use DomEdit::*;
 #[test]
 fn html_and_rsx_generate_the_same_output() {
     let dom = new_dom();
-    let (create, change) = dom.lazy_diff(
+    let (create, change) = dom.diff_lazynodes(
         rsx! ( div { "Hello world" } ),
         rsx! ( div { "Goodbye world" } ),
     );
     assert_eq!(
         create.edits,
         [
-            CreateElement { id: 0, tag: "div" },
+            CreateElement {
+                root: 1,
+                tag: "div"
+            },
             CreateTextNode {
-                id: 1,
+                root: 2,
                 text: "Hello world"
             },
             AppendChildren { many: 1 },
@@ -42,13 +46,10 @@ fn html_and_rsx_generate_the_same_output() {
 
     assert_eq!(
         change.edits,
-        [
-            PushRoot { id: 1 },
-            SetText {
-                text: "Goodbye world"
-            },
-            PopRoot
-        ]
+        [SetText {
+            text: "Goodbye world",
+            root: 2
+        },]
     );
 }
 
@@ -57,7 +58,7 @@ fn html_and_rsx_generate_the_same_output() {
 fn fragments_create_properly() {
     let dom = new_dom();
 
-    let create = dom.create(rsx! {
+    let create = dom.create_vnodes(rsx! {
         div { "Hello a" }
         div { "Hello b" }
         div { "Hello c" }
@@ -66,21 +67,30 @@ fn fragments_create_properly() {
     assert_eq!(
         create.edits,
         [
-            CreateElement { id: 0, tag: "div" },
+            CreateElement {
+                root: 1,
+                tag: "div"
+            },
             CreateTextNode {
-                id: 1,
+                root: 2,
                 text: "Hello a"
             },
             AppendChildren { many: 1 },
-            CreateElement { id: 2, tag: "div" },
+            CreateElement {
+                root: 3,
+                tag: "div"
+            },
             CreateTextNode {
-                id: 3,
+                root: 4,
                 text: "Hello b"
             },
             AppendChildren { many: 1 },
-            CreateElement { id: 4, tag: "div" },
+            CreateElement {
+                root: 5,
+                tag: "div"
+            },
             CreateTextNode {
-                id: 5,
+                root: 6,
                 text: "Hello c"
             },
             AppendChildren { many: 1 },
@@ -97,17 +107,20 @@ fn empty_fragments_create_anchors() {
     let left = rsx!({ (0..0).map(|_f| rsx! { div {}}) });
     let right = rsx!({ (0..1).map(|_f| rsx! { div {}}) });
 
-    let (create, change) = dom.lazy_diff(left, right);
+    let (create, change) = dom.diff_lazynodes(left, right);
 
     assert_eq!(
         create.edits,
-        [CreatePlaceholder { id: 0 }, AppendChildren { many: 1 }]
+        [CreatePlaceholder { root: 1 }, AppendChildren { many: 1 }]
     );
     assert_eq!(
         change.edits,
         [
-            CreateElement { id: 1, tag: "div" },
-            ReplaceWith { m: 1, root: 0 }
+            CreateElement {
+                root: 2,
+                tag: "div"
+            },
+            ReplaceWith { m: 1, root: 1 }
         ]
     );
 }
@@ -120,20 +133,35 @@ fn empty_fragments_create_many_anchors() {
     let left = rsx!({ (0..0).map(|_f| rsx! { div {}}) });
     let right = rsx!({ (0..5).map(|_f| rsx! { div {}}) });
 
-    let (create, change) = dom.lazy_diff(left, right);
+    let (create, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         create.edits,
-        [CreatePlaceholder { id: 0 }, AppendChildren { many: 1 }]
+        [CreatePlaceholder { root: 1 }, AppendChildren { many: 1 }]
     );
     assert_eq!(
         change.edits,
         [
-            CreateElement { id: 1, tag: "div" },
-            CreateElement { id: 2, tag: "div" },
-            CreateElement { id: 3, tag: "div" },
-            CreateElement { id: 4, tag: "div" },
-            CreateElement { id: 5, tag: "div" },
-            ReplaceWith { m: 5, root: 0 }
+            CreateElement {
+                root: 2,
+                tag: "div"
+            },
+            CreateElement {
+                root: 3,
+                tag: "div"
+            },
+            CreateElement {
+                root: 4,
+                tag: "div"
+            },
+            CreateElement {
+                root: 5,
+                tag: "div"
+            },
+            CreateElement {
+                root: 6,
+                tag: "div"
+            },
+            ReplaceWith { m: 5, root: 1 }
         ]
     );
 }
@@ -151,33 +179,43 @@ fn empty_fragments_create_anchors_with_many_children() {
         })
     });
 
-    let (create, change) = dom.lazy_diff(left, right);
+    let (create, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         create.edits,
-        [CreatePlaceholder { id: 0 }, AppendChildren { many: 1 }]
+        [CreatePlaceholder { root: 1 }, AppendChildren { many: 1 }]
     );
+
     assert_eq!(
         change.edits,
         [
-            CreateElement { id: 1, tag: "div" },
+            CreateElement {
+                tag: "div",
+                root: 2,
+            },
             CreateTextNode {
                 text: "hello: 0",
-                id: 2
+                root: 3
             },
             AppendChildren { many: 1 },
-            CreateElement { id: 3, tag: "div" },
+            CreateElement {
+                tag: "div",
+                root: 4,
+            },
             CreateTextNode {
                 text: "hello: 1",
-                id: 4
+                root: 5
             },
             AppendChildren { many: 1 },
-            CreateElement { id: 5, tag: "div" },
+            CreateElement {
+                tag: "div",
+                root: 6,
+            },
             CreateTextNode {
                 text: "hello: 2",
-                id: 6
+                root: 7
             },
             AppendChildren { many: 1 },
-            ReplaceWith { m: 3, root: 0 }
+            ReplaceWith { root: 1, m: 3 }
         ]
     );
 }
@@ -194,33 +232,38 @@ fn many_items_become_fragment() {
     });
     let right = rsx!({ (0..0).map(|_| rsx! { div {} }) });
 
-    let (create, change) = dom.lazy_diff(left, right);
+    let (create, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         create.edits,
         [
-            CreateElement { id: 0, tag: "div" },
+            CreateElement {
+                root: 1,
+                tag: "div"
+            },
             CreateTextNode {
                 text: "hello",
-                id: 1
+                root: 2
             },
             AppendChildren { many: 1 },
-            CreateElement { id: 2, tag: "div" },
+            CreateElement {
+                root: 3,
+                tag: "div"
+            },
             CreateTextNode {
                 text: "hello",
-                id: 3
+                root: 4
             },
             AppendChildren { many: 1 },
             AppendChildren { many: 2 },
         ]
     );
 
-    // hmmmmmmmmm worried about reusing IDs that we shouldnt be
     assert_eq!(
         change.edits,
         [
-            Remove { root: 2 },
-            CreatePlaceholder { id: 4 },
-            ReplaceWith { root: 0, m: 1 },
+            CreatePlaceholder { root: 5 },
+            ReplaceWith { root: 1, m: 1 },
+            Remove { root: 3 },
         ]
     );
 }
@@ -241,7 +284,7 @@ fn two_equal_fragments_are_equal() {
         })
     });
 
-    let (_create, change) = dom.lazy_diff(left, right);
+    let (_create, change) = dom.diff_lazynodes(left, right);
     assert!(change.edits.is_empty());
 }
 
@@ -259,21 +302,21 @@ fn two_fragments_with_differrent_elements_are_differet() {
         p {}
     );
 
-    let (_create, changes) = dom.lazy_diff(left, right);
+    let (_create, changes) = dom.diff_lazynodes(left, right);
     log::debug!("{:#?}", &changes);
     assert_eq!(
         changes.edits,
         [
             // create the new h1s
-            CreateElement { tag: "h1", id: 3 },
-            CreateElement { tag: "h1", id: 4 },
-            CreateElement { tag: "h1", id: 5 },
-            InsertAfter { root: 1, n: 3 },
+            CreateElement { tag: "h1", root: 4 },
+            CreateElement { tag: "h1", root: 5 },
+            CreateElement { tag: "h1", root: 6 },
+            InsertAfter { root: 2, n: 3 },
             // replace the divs with new h1s
-            CreateElement { tag: "h1", id: 6 },
-            ReplaceWith { root: 0, m: 1 },
-            CreateElement { tag: "h1", id: 7 },
+            CreateElement { tag: "h1", root: 7 },
             ReplaceWith { root: 1, m: 1 },
+            CreateElement { tag: "h1", root: 8 },
+            ReplaceWith { root: 2, m: 1 },
         ]
     );
 }
@@ -292,29 +335,44 @@ fn two_fragments_with_differrent_elements_are_differet_shorter() {
         p {}
     );
 
-    let (create, change) = dom.lazy_diff(left, right);
+    let (create, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         create.edits,
         [
-            CreateElement { id: 0, tag: "div" },
-            CreateElement { id: 1, tag: "div" },
-            CreateElement { id: 2, tag: "div" },
-            CreateElement { id: 3, tag: "div" },
-            CreateElement { id: 4, tag: "div" },
-            CreateElement { id: 5, tag: "p" },
+            CreateElement {
+                root: 1,
+                tag: "div"
+            },
+            CreateElement {
+                root: 2,
+                tag: "div"
+            },
+            CreateElement {
+                root: 3,
+                tag: "div"
+            },
+            CreateElement {
+                root: 4,
+                tag: "div"
+            },
+            CreateElement {
+                root: 5,
+                tag: "div"
+            },
+            CreateElement { root: 6, tag: "p" },
             AppendChildren { many: 6 },
         ]
     );
     assert_eq!(
         change.edits,
         [
-            Remove { root: 2 },
             Remove { root: 3 },
             Remove { root: 4 },
-            CreateElement { id: 6, tag: "h1" },
-            ReplaceWith { root: 0, m: 1 },
-            CreateElement { id: 7, tag: "h1" },
+            Remove { root: 5 },
+            CreateElement { root: 7, tag: "h1" },
             ReplaceWith { root: 1, m: 1 },
+            CreateElement { root: 8, tag: "h1" },
+            ReplaceWith { root: 2, m: 1 },
         ]
     );
 }
@@ -333,23 +391,38 @@ fn two_fragments_with_same_elements_are_differet() {
         p {}
     );
 
-    let (create, change) = dom.lazy_diff(left, right);
+    let (create, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         create.edits,
         [
-            CreateElement { id: 0, tag: "div" },
-            CreateElement { id: 1, tag: "div" },
-            CreateElement { id: 2, tag: "p" },
+            CreateElement {
+                root: 1,
+                tag: "div"
+            },
+            CreateElement {
+                root: 2,
+                tag: "div"
+            },
+            CreateElement { root: 3, tag: "p" },
             AppendChildren { many: 3 },
         ]
     );
     assert_eq!(
         change.edits,
         [
-            CreateElement { id: 3, tag: "div" },
-            CreateElement { id: 4, tag: "div" },
-            CreateElement { id: 5, tag: "div" },
-            InsertAfter { root: 1, n: 3 },
+            CreateElement {
+                root: 4,
+                tag: "div"
+            },
+            CreateElement {
+                root: 5,
+                tag: "div"
+            },
+            CreateElement {
+                root: 6,
+                tag: "div"
+            },
+            InsertAfter { root: 2, n: 3 },
         ]
     );
 }
@@ -364,14 +437,14 @@ fn keyed_diffing_order() {
         p {"e"}
     );
     let right = rsx!(
-        {(0..2).map(|f| {rsx! { div { key: "{f}" }}})}
+        {(0..2).map(|f| rsx! { div { key: "{f}" }})}
         p {"e"}
     );
 
-    let (create, change) = dom.lazy_diff(left, right);
+    let (create, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         change.edits,
-        [Remove { root: 2 }, Remove { root: 3 }, Remove { root: 4 },]
+        [Remove { root: 3 }, Remove { root: 4 }, Remove { root: 5 },]
     );
 }
 
@@ -392,11 +465,11 @@ fn keyed_diffing_out_of_order() {
         })
     });
 
-    let (_, changes) = dom.lazy_diff(left, right);
+    let (_, changes) = dom.diff_lazynodes(left, right);
     log::debug!("{:?}", &changes);
     assert_eq!(
         changes.edits,
-        [PushRoot { id: 6 }, InsertBefore { root: 4, n: 1 }]
+        [PushRoot { root: 7 }, InsertBefore { root: 5, n: 1 }]
     );
 }
 
@@ -417,13 +490,13 @@ fn keyed_diffing_out_of_order_adds() {
         })
     });
 
-    let (_, change) = dom.lazy_diff(left, right);
+    let (_, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         change.edits,
         [
-            PushRoot { id: 4 },
-            PushRoot { id: 3 },
-            InsertBefore { n: 2, root: 0 }
+            PushRoot { root: 5 },
+            PushRoot { root: 4 },
+            InsertBefore { n: 2, root: 1 }
         ]
     );
 }
@@ -444,13 +517,13 @@ fn keyed_diffing_out_of_order_adds_2() {
         })
     });
 
-    let (_, change) = dom.lazy_diff(left, right);
+    let (_, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         change.edits,
         [
-            PushRoot { id: 3 },
-            PushRoot { id: 4 },
-            InsertBefore { n: 2, root: 0 }
+            PushRoot { root: 4 },
+            PushRoot { root: 5 },
+            InsertBefore { n: 2, root: 1 }
         ]
     );
 }
@@ -472,13 +545,13 @@ fn keyed_diffing_out_of_order_adds_3() {
         })
     });
 
-    let (_, change) = dom.lazy_diff(left, right);
+    let (_, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         change.edits,
         [
-            PushRoot { id: 4 },
-            PushRoot { id: 3 },
-            InsertBefore { n: 2, root: 1 }
+            PushRoot { root: 5 },
+            PushRoot { root: 4 },
+            InsertBefore { n: 2, root: 2 }
         ]
     );
 }
@@ -500,13 +573,13 @@ fn keyed_diffing_out_of_order_adds_4() {
         })
     });
 
-    let (_, change) = dom.lazy_diff(left, right);
+    let (_, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         change.edits,
         [
-            PushRoot { id: 4 },
-            PushRoot { id: 3 },
-            InsertBefore { n: 2, root: 2 }
+            PushRoot { root: 5 },
+            PushRoot { root: 4 },
+            InsertBefore { n: 2, root: 3 }
         ]
     );
 }
@@ -528,10 +601,10 @@ fn keyed_diffing_out_of_order_adds_5() {
         })
     });
 
-    let (_, change) = dom.lazy_diff(left, right);
+    let (_, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         change.edits,
-        [PushRoot { id: 4 }, InsertBefore { n: 1, root: 3 }]
+        [PushRoot { root: 5 }, InsertBefore { n: 1, root: 4 }]
     );
 }
 
@@ -551,13 +624,19 @@ fn keyed_diffing_additions() {
         })
     });
 
-    let (_, change) = dom.lazy_diff(left, right);
+    let (_, change) = dom.diff_lazynodes(left, right);
     assert_eq!(
         change.edits,
         [
-            CreateElement { id: 5, tag: "div" },
-            CreateElement { id: 6, tag: "div" },
-            InsertAfter { n: 2, root: 4 }
+            CreateElement {
+                root: 6,
+                tag: "div"
+            },
+            CreateElement {
+                root: 7,
+                tag: "div"
+            },
+            InsertAfter { n: 2, root: 5 }
         ]
     );
 }
@@ -578,18 +657,24 @@ fn keyed_diffing_additions_and_moves_on_ends() {
         })
     });
 
-    let (_, change) = dom.lazy_diff(left, right);
+    let (_, change) = dom.diff_lazynodes(left, right);
     log::debug!("{:?}", change);
     assert_eq!(
         change.edits,
         [
             // create 11, 12
-            CreateElement { tag: "div", id: 4 },
-            CreateElement { tag: "div", id: 5 },
-            InsertAfter { root: 2, n: 2 },
+            CreateElement {
+                tag: "div",
+                root: 5
+            },
+            CreateElement {
+                tag: "div",
+                root: 6
+            },
+            InsertAfter { root: 3, n: 2 },
             // move 7 to the front
-            PushRoot { id: 3 },
-            InsertBefore { root: 0, n: 1 }
+            PushRoot { root: 4 },
+            InsertBefore { root: 1, n: 1 }
         ]
     );
 }
@@ -611,22 +696,34 @@ fn keyed_diffing_additions_and_moves_in_middle() {
     });
 
     // LIS: 4, 5, 6
-    let (_, change) = dom.lazy_diff(left, right);
+    let (_, change) = dom.diff_lazynodes(left, right);
     log::debug!("{:#?}", change);
     assert_eq!(
         change.edits,
         [
             // create 13, 17
-            CreateElement { tag: "div", id: 4 },
-            CreateElement { tag: "div", id: 5 },
-            InsertBefore { root: 1, n: 2 },
-            // create 11, 12
-            CreateElement { tag: "div", id: 6 },
-            CreateElement { tag: "div", id: 7 },
+            CreateElement {
+                tag: "div",
+                root: 5
+            },
+            CreateElement {
+                tag: "div",
+                root: 6
+            },
             InsertBefore { root: 2, n: 2 },
+            // create 11, 12
+            CreateElement {
+                tag: "div",
+                root: 7
+            },
+            CreateElement {
+                tag: "div",
+                root: 8
+            },
+            InsertBefore { root: 3, n: 2 },
             // move 7
-            PushRoot { id: 3 },
-            InsertBefore { root: 0, n: 1 }
+            PushRoot { root: 4 },
+            InsertBefore { root: 1, n: 1 }
         ]
     );
 }
@@ -648,22 +745,28 @@ fn controlled_keyed_diffing_out_of_order() {
     });
 
     // LIS: 5, 6
-    let (_, changes) = dom.lazy_diff(left, right);
+    let (_, changes) = dom.diff_lazynodes(left, right);
     log::debug!("{:#?}", &changes);
     assert_eq!(
         changes.edits,
         [
             // move 4 to after 6
-            PushRoot { id: 0 },
-            InsertAfter { n: 1, root: 2 },
+            PushRoot { root: 1 },
+            InsertAfter { n: 1, root: 3 },
             // remove 7
 
             // create 9 and insert before 6
-            CreateElement { id: 4, tag: "div" },
-            InsertBefore { n: 1, root: 2 },
+            CreateElement {
+                root: 5,
+                tag: "div"
+            },
+            InsertBefore { n: 1, root: 3 },
             // create 0 and insert before 5
-            CreateElement { id: 5, tag: "div" },
-            InsertBefore { n: 1, root: 1 },
+            CreateElement {
+                root: 6,
+                tag: "div"
+            },
+            InsertBefore { n: 1, root: 2 },
         ]
     );
 }
@@ -684,33 +787,83 @@ fn controlled_keyed_diffing_out_of_order_max_test() {
         })
     });
 
-    let (_, changes) = dom.lazy_diff(left, right);
+    let (_, changes) = dom.diff_lazynodes(left, right);
     log::debug!("{:#?}", &changes);
     assert_eq!(
         changes.edits,
         [
-            CreateElement { id: 5, tag: "div" },
-            InsertBefore { n: 1, root: 2 },
-            PushRoot { id: 3 },
-            InsertBefore { n: 1, root: 0 },
+            CreateElement {
+                root: 6,
+                tag: "div"
+            },
+            InsertBefore { n: 1, root: 3 },
+            PushRoot { root: 4 },
+            InsertBefore { n: 1, root: 1 },
         ]
     );
 }
 
+// noticed some weird behavior in the desktop interpreter
+// just making sure it doesnt happen in the core implementation
 #[test]
-fn suspense() {
+fn remove_list() {
     let dom = new_dom();
 
-    let edits = dom.create(LazyNodes::new(|f| {
-        use std::cell::{Cell, RefCell};
-        VNode::Suspended(f.bump().alloc(VSuspended {
-            task_id: 0,
-            callback: RefCell::new(None),
-            dom_id: Cell::new(None),
-        }))
-    }));
+    let left = rsx!({
+        (0..10).rev().take(5).map(|i| {
+            rsx! { Fragment { key: "{i}", "{i}" }}
+        })
+    });
+
+    let right = rsx!({
+        (0..10).rev().take(2).map(|i| {
+            rsx! { Fragment { key: "{i}", "{i}" }}
+        })
+    });
+
+    let (create, changes) = dom.diff_lazynodes(left, right);
+
+    // dbg!(create);
+    // dbg!(changes);
+
     assert_eq!(
-        edits.edits,
-        [CreatePlaceholder { id: 0 }, AppendChildren { many: 1 }]
+        changes.edits,
+        [
+            // remove 5, 4, 3
+            Remove { root: 3 },
+            Remove { root: 4 },
+            Remove { root: 5 },
+        ]
+    );
+}
+
+// noticed some weird behavior in the desktop interpreter
+// just making sure it doesnt happen in the core implementation
+#[test]
+fn remove_list_nokeyed() {
+    let dom = new_dom();
+
+    let left = rsx!({
+        (0..10).rev().take(5).map(|i| {
+            rsx! { Fragment { "{i}" }}
+        })
+    });
+
+    let right = rsx!({
+        (0..10).rev().take(2).map(|i| {
+            rsx! { Fragment { "{i}" }}
+        })
+    });
+
+    let (create, changes) = dom.diff_lazynodes(left, right);
+
+    assert_eq!(
+        changes.edits,
+        [
+            // remove 5, 4, 3
+            Remove { root: 3 },
+            Remove { root: 4 },
+            Remove { root: 5 },
+        ]
     );
 }
