@@ -1,4 +1,6 @@
-use dioxus::{nodes::VSuspended, prelude::*, DomEdit, TestDom};
+#![allow(unused, non_upper_case_globals)]
+
+use dioxus::{prelude::*, DomEdit, Mutations};
 use dioxus_core as dioxus;
 use dioxus_core_macro::*;
 use dioxus_html as dioxus_elements;
@@ -7,24 +9,18 @@ use DomEdit::*;
 
 mod test_logging;
 
-fn new_dom() -> TestDom {
-    const IS_LOGGING_ENABLED: bool = false;
-    test_logging::set_up_logging(IS_LOGGING_ENABLED);
-    TestDom::new()
-}
-
 #[test]
 fn shared_state_test() {
     struct MySharedState(&'static str);
 
-    static App: FC<()> = |cx, props| {
-        cx.use_provide_state(|| MySharedState("world!"));
-        rsx!(cx, Child {})
+    static App: Component = |cx| {
+        cx.provide_context(MySharedState("world!"));
+        cx.render(rsx!(Child {}))
     };
 
-    static Child: FC<()> = |cx, props| {
-        let shared = cx.use_consume_state::<MySharedState>()?;
-        rsx!(cx, "Hello, {shared.0}")
+    static Child: Component = |cx| {
+        let shared = cx.consume_context::<MySharedState>()?;
+        cx.render(rsx!("Hello, {shared.0}"))
     };
 
     let mut dom = VirtualDom::new(App);
@@ -34,7 +30,7 @@ fn shared_state_test() {
         edits,
         [
             CreateTextNode {
-                id: 0,
+                root: 1,
                 text: "Hello, world!"
             },
             AppendChildren { many: 1 },
